@@ -22,7 +22,6 @@ import glob
 import json
 import logging
 import os
-import sys
 import subprocess
 import sys
 import tempfile
@@ -31,6 +30,7 @@ import multiprocessing
 
 logger = None
 
+
 def get_logger():
     global logger
     if logger is None:
@@ -83,16 +83,12 @@ def pull_image(name):
         log.debug(cmd_stderr)
 
 
-def match_config_volume(prefix, config):
-    # Match the mounted config volume - we can't just use the
+def match_config_volumes(prefix, config):
+    # Match the mounted config volumes - we can't just use the
     # key as e.g "novacomute" consumes config-data/nova
     volumes = config.get('volumes', [])
-    config_volume=None
-    for v in volumes:
-        if v.startswith(prefix):
-            config_volume = os.path.dirname(v.split(":")[0])
-            break
-    return config_volume
+    return sorted([os.path.dirname(v.split(":")[0]) for v in volumes if
+                   v.startswith(prefix)])
 
 
 def get_config_hash(config_volume):
@@ -135,6 +131,8 @@ process_count = int(os.environ.get('PROCESS_COUNT',
 log = get_logger()
 log.info('Running docker-puppet')
 config_file = os.environ.get('CONFIG', '/var/lib/docker-puppet/docker-puppet.json')
+# If specified, only this config_volume will be used
+config_volume_only = os.environ.get('CONFIG_VOLUME', None)
 log.debug('CONFIG: %s' % config_file)
 with open(config_file) as f:
     json_data = json.load(f)
@@ -172,15 +170,15 @@ for service in (json_data or []):
     if not manifest or not config_image:
         continue
 
-    log.info('config_volume %s' % config_volume)
-    log.info('puppet_tags %s' % puppet_tags)
-    log.info('manifest %s' % manifest)
-    log.info('config_image %s' % config_image)
-    log.info('volumes %s' % volumes)
+    log.debug('config_volume %s' % config_volume)
+    log.debug('puppet_tags %s' % puppet_tags)
+    log.debug('manifest %s' % manifest)
+    log.debug('config_image %s' % config_image)
+    log.debug('volumes %s' % volumes)
     # We key off of config volume for all configs.
     if config_volume in configs:
         # Append puppet tags and manifest.
-        log.info("Existing service, appending puppet tags and manifest")
+        log.debug("Existing service, appending puppet tags and manifest")
         if puppet_tags:
             configs[config_volume][1] = '%s,%s' % (configs[config_volume][1],
                                                    puppet_tags)
@@ -190,15 +188,24 @@ for service in (json_data or []):
         if configs[config_volume][3] != config_image:
             log.warn("Config containers do not match even though"
                      " shared volumes are the same!")
+        if volumes:
+            configs[config_volume][4].extend(volumes)
+
     else:
-        log.info("Adding new service")
-        configs[config_volume] = service
+        if not config_volume_only or (config_volume_only == config_volume):
+            log.debug("Adding new service")
+            configs[config_volume] = service
+        else:
+            log.debug("Ignoring %s due to $CONFIG_VOLUME=%s" %
+                (config_volume, config_volume_only))
 
 log.info('Service compilation completed.')
 
+
 def mp_puppet_config((config_volume, puppet_tags, manifest, config_image, volumes)):
     log = get_logger()
-    log.info('Started processing puppet configs')
+    log.info('Starting configuration of %s using image %s' % (config_volume,
+             config_image))
     log.debug('config_volume %s' % config_volume)
     log.debug('puppet_tags %s' % puppet_tags)
     log.debug('manifest %s' % manifest)
@@ -227,7 +234,7 @@ def mp_puppet_config((config_volume, puppet_tags, manifest, config_image, volume
         sync
 
         set +e
-        FACTER_hostname=$HOSTNAME FACTER_uuid=docker /usr/bin/puppet apply \
+        FACTER_hostname=$HOSTNAME FACTER_uuid=docker /usr/bin/puppet apply --summarize \
         --detailed-exitcodes --color=false --logdest syslog --logdest console --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules $TAGS /etc/config.pp
         rc=$?
         set -e
@@ -273,17 +280,16 @@ def mp_puppet_config((config_volume, puppet_tags, manifest, config_image, volume
         dcmd = ['/usr/bin/docker', 'run',
                 '--user', 'root',
                 '--name', 'docker-puppet-%s' % config_volume,
-                '--health-cmd', '/bin/true',
                 '--env', 'PUPPET_TAGS=%s' % puppet_tags,
                 '--env', 'NAME=%s' % config_volume,
                 '--env', 'HOSTNAME=%s' % short_hostname(),
                 '--env', 'NO_ARCHIVE=%s' % os.environ.get('NO_ARCHIVE', ''),
                 '--env', 'STEP=%s' % os.environ.get('STEP', '6'),
                 '--volume', '/etc/localtime:/etc/localtime:ro',
-                '--volume', '%s:/etc/config.pp:ro' % tmp_man.name,
-                '--volume', '/etc/puppet/:/tmp/puppet-etc/:ro',
-                '--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro',
-                '--volume', '%s:/var/lib/config-data/:rw' % os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data'),
+                '--volume', '%s:/etc/config.pp:ro,z' % tmp_man.name,
+                '--volume', '/etc/puppet/:/tmp/puppet-etc/:ro,z',
+                '--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro,z',
+                '--volume', '%s:/var/lib/config-data/:z' % os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data'),
                 '--volume', 'tripleo_logs:/var/log/tripleo/',
                 # Syslog socket for puppet logs
                 '--volume', '/dev/log:/dev/log',
@@ -293,7 +299,7 @@ def mp_puppet_config((config_volume, puppet_tags, manifest, config_image, volume
                 '--volume', '/etc/pki/tls/certs/ca-bundle.trust.crt:/etc/pki/tls/certs/ca-bundle.trust.crt:ro',
                 '--volume', '/etc/pki/tls/cert.pem:/etc/pki/tls/cert.pem:ro',
                 # script injection
-                '--volume', '%s:%s:rw' % (sh_script, sh_script) ]
+                '--volume', '%s:%s:z' % (sh_script, sh_script) ]
 
         for volume in volumes:
             if volume:
@@ -333,7 +339,7 @@ def mp_puppet_config((config_volume, puppet_tags, manifest, config_image, volume
             # only delete successful runs, for debugging
             rm_container('docker-puppet-%s' % config_volume)
 
-        log.info('Finished processing puppet configs')
+        log.info('Finished processing puppet configs for %s' % (config_volume))
         return subproc.returncode
 
 # Holds all the information for each process to consume.
@@ -362,6 +368,8 @@ for p in process_map:
 
 # Fire off processes to perform each configuration.  Defaults
 # to the number of CPUs on the system.
+log.info('Starting multiprocess configuration steps.  Using %d processes.' %
+         process_count)
 p = multiprocessing.Pool(process_count)
 returncodes = list(p.map(mp_puppet_config, process_map))
 config_volumes = [pm[0] for pm in process_map]
@@ -383,19 +391,20 @@ for infile in infiles:
         infile_data = json.load(f)
 
     for k, v in infile_data.iteritems():
-        config_volume = match_config_volume(config_volume_prefix, v)
-        if config_volume:
-            config_hash = get_config_hash(config_volume)
-            if config_hash:
-                env = v.get('environment', [])
-                env.append("TRIPLEO_CONFIG_HASH=%s" % config_hash)
-                log.debug("Updating config hash for %s, config_volume=%s hash=%s" % (k, config_volume, config_hash))
-                infile_data[k]['environment'] = env
+        config_volumes = match_config_volumes(config_volume_prefix, v)
+        config_hashes = [get_config_hash(volume_path) for volume_path in config_volumes]
+        config_hashes = filter(None, config_hashes)
+        config_hash = '-'.join(config_hashes)
+        if config_hash:
+            env = v.get('environment', [])
+            env.append("TRIPLEO_CONFIG_HASH=%s" % config_hash)
+            log.debug("Updating config hash for %s, config_volume=%s hash=%s" % (k, config_volume, config_hash))
+            infile_data[k]['environment'] = env
 
     outfile = os.path.join(os.path.dirname(infile), "hashed-" + os.path.basename(infile))
     with open(outfile, 'w') as out_f:
         os.chmod(out_f.name, 0600)
-        json.dump(infile_data, out_f)
+        json.dump(infile_data, out_f, indent=2)
 
 if not success:
     sys.exit(1)
